class ResourcePicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  has_many :panel_pictures
  belongs_to :original_picture
  
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :artist_id, :presence => true, :numericality => true, :existence => true
  validates :license_id, :presence => true, :numericality => true, :existence => true
  validates :original_picture_id, :presence => true, :numericality => true, :existence => true
  
  before_destroy :destroy_with_file
  
  def destroy_with_file
    PictureIO.resource_picture_io.delete self.filename
    PictureIO.resource_picture_io.class.subdirs.each do |d|
      next if d.empty?
      PictureIO.resource_picture_io.delete(self.filename, d) if PictureIO.resource_picture_io.exist?(self.filename, d)
    end
  end
  
  def self.resize(data, dw, dh)
    Magick::Image.from_blob(data).shift.resize(dw, dh)
  end
  
  #サイズの調整（limw,limhに必ず収まるように合わせる）
  def self.fix_size_both(limw, limh, w, h)
    wr = if w > limw
      limw*100/w
    else
      100
    end
    hr = if h > limh
      limh*100/h
    else
      100
    end
    res = if wr < hr
      #幅の方が圧縮率が高い
      [w*wr/100, h*wr/100]
    else
      #高さの方が圧縮率が高い
      [w*hr/100, h*hr/100]
    end
    res
  end
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def v(rimg)
    rimg.flip.to_blob
  end
  
  def h(rimg)
    rimg.flop.to_blob
  end
  
  def vh(rimg)
    rimg.flip.flop.to_blob
  end
  
  def url subdir = nil
    '/resource_pictures/' + (subdir.to_s.empty? ? '' : subdir.to_s + '/' ) + filename
  end
  
  def thumbnail(rimg)
    tw, th = ResourcePicture.fix_size_both(64, 64, rimg.columns, rimg.rows)
    ResourcePicture.resize(rimg.to_blob, tw, th).to_blob
  end
  
  def self.update_picture(op)
    res = op.resource_picture || ResourcePicture.new
    res.attributes = {:width => op.width, :height => op.height, :ext => op.ext, :filesize => op.filesize, 
      :original_picture_id => op.id, :artist_id => op.artist_id, :license_id => op.license_id
    }
    res
  end
  
  def to_gif?
    self.dext == 'png' and self.license.no_convert == 0
  end
  
  def self.png_to_gif(data)
    res = nil
    begin
      mgk = Magick::Image.from_blob(data).shift
      mgk.format = 'gif'
      res = mgk
    rescue
      res = false
    end
    res
  end
  
  def store(mgk)
    res = false
    if res = self.save
      if res = self.store_picture(mgk)
        if self.to_gif?
          if gifmgk = ResourcePicture.png_to_gif(mgk.to_blob)
            res = self.store_picture(gifmgk)
          else
            res = false
          end
        end
      end
    end
    res
  end
  
  def store_picture(mgk)
    res = false
    PictureIO.resource_picture_io.class.subdirs.each do |d|
      picdata = d.empty? ? mgk.to_blob : self.__send__(d, mgk)
      res = PictureIO.resource_picture_io.put(picdata, "#{self.id}.#{mgk.format}", d)
      break unless res
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    ResourcePicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => [:license, :artist]}
  end
  
  def self.list_json_opt
    {:include => [:license, :artist]}
  end
  
  def self.show rid, opt = {}
    r = ResourcePicture.find(rid, :include => self.show_include_opt(opt))
#    raise ActiveRecord::Forbidden unless c.visible?(au)
    r
  end
  
  def self.show_include_opt opt = {}
    res = [:license, :artist]
    res.push(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => [:license, :artist]}
  end
  
  def self.visible_count
    ResourcePicture.count
  end
  
end
